///////////////////////////////////////////////////
// bookmark duplicate detector v0.0.1 by Paradox
// v0.0.5-0.0.7 modified for firefox 1.5.0 rc1 by Stefano Caselli
// Since v0.1.4 Maintainer is Stphane BERTIN
//	- Drag&drop support and 'parent folder',
//	- Bookmark this link,
//	- Bookmark All Tabs,
//	- Search duplicate URL
// v0.2.2 --> solve Firefox 1.0 bug in search function
//TODO Be careful with URL url != url/ and so on !!!!

//Init Bookmarks.js Service
initServices();
initBMService();

// Replace BookmarksUtils method by our functions to detect duplicate URL
BookmarksUtils.insertAndCheckSelection = insertAndCheckURL; 
BookmarksUtils.addBookmark = addBookmarkURL; 

//gNC_NS is defined since Firefox 1.5 --> NC_NS === undefined
//NC_NS is defined for Firefox 1.0 --> gNC_NS === undefined
const gNCNS = "http://home.netscape.com/NC-rdf#";
var version = "1.5";
try {
	if (gNC_NS) version="1.5";
}
catch(e){
	version = "1.0";
}

// Used by drag&drop
function insertAndCheckURL (aAction, aSelection, aTarget, aTargetIndex)
{
	// Used to DEBUG
	var DEBUG = false;
	var DuplicatesBundle = document.getElementById('duplicates-strings');
	var isValid = BookmarksUtils.isSelectionValidForInsertion(aSelection, aTarget);
	if (!isValid) {
		SOUND.beep();
		return false;
	}
	if (aAction == "drag" || aAction == "paste"){
		var itemValue = BMDS.GetTarget(aSelection.item[0], RDF.GetResource(gNCNS+"URL"), true);
		var url = itemValue.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;

		var isDuplicate = isBookmarked(url);
		if(isDuplicate) {
			var message = DuplicatesBundle.getString("alreadyIn-Folder")+"\n";
			message += isDuplicate.join("\n")+"\n";
			message += DuplicatesBundle.getString("alreadyIn-Confirm");
			if(! confirm(message)) return false;
		}
	} else if (DEBUG) { alert("DEBUG: insertAndCheckURL + aAction: "+aAction);}

	BookmarksUtils.insertSelection(aAction, aSelection, aTarget, aTargetIndex);
	BookmarksUtils.flushDataSource();
	if (version == "1.5") BookmarksUtils.refreshSearch();
	return true;
}

// Verify if the specified URL is already in Bookmark
// If URL already in Bookmark function return an Array of Parent folder
function isBookmarked(url)
{
	var DuplicatesBundle = document.getElementById('duplicates-strings');

	if ( ! BMSVC || ! BMDS || ! RDF || ! gNCNS )
	{
		alert(DuplicatesBundle.getString("flagrantError-Alert"));
		return false; 
	}

	var urlLiteral= RDF.GetLiteral(url);
	var bmResource = BMSVC.GetSources(RDF.GetResource(gNCNS+"URL"), urlLiteral, true);
	var count = 0, URLParentName = new Array();
	while (bmResource.hasMoreElements()) {
		var DuplicateURLresource=bmResource.getNext();
		var URLParent = BMSVC.getParent(DuplicateURLresource);
		if(URLParent) {
			//URLParentName[count] = BookmarksUtils.getProperty(URLParent.Value,gNCNS+"Name");
			URLParentName[count] = findPath(DuplicateURLresource);
			count++;
		}
	}

	if (count>0) return URLParentName;
	return false;
}

// Replace addBookmark to verify if URL is duplicated
function addBookmarkURL (aURL, aTitle, aCharset, aIsWebPanel, aDescription)
{
	var DuplicatesBundle = document.getElementById('duplicates-strings');
	var dArgs = {
		name: aTitle,
		url: aURL,
		charset: aCharset,
		bWebPanel: aIsWebPanel,
		description: aDescription
	}

	var isDuplicate = isBookmarked(aURL);
	if(isDuplicate) {
		var message = DuplicatesBundle.getString("alreadyIn-Folder")+"\n";
		message += isDuplicate.join("\n")+"\n";
		message += DuplicatesBundle.getString("alreadyIn-Confirm");
		if(! confirm(message)) return false;
	}

    if(version == "1.5") {
		return openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", ADD_BM_DIALOG_FEATURES, dArgs);
	} else if (version == "1.0") {
		return openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", "centerscreen,chrome,dialog,resizable,dependent", aTitle,aURL,null,aCharset,null,null,aIsWebPanel);
	}

}

// Function used to search in ALL Bookmarks which URL are duplicated
function searchDuplicateURL ()
{
	var DuplicatesBundle = document.getElementById('duplicates-strings');
	var urlArc   = RDF.GetResource(gNCNS+"URL");

	try {
		var AllBookmarksResources = BMDS.GetAllResources();
	} catch(err) {
		Components.utils.reportError(err);
	}

	var containerChildren = AllBookmarksResources;
	var AllURLs = new Array();
	var AllURLsTotal = 0;
	while (containerChildren.hasMoreElements()) {
		var urlRes = containerChildren.getNext().QueryInterface(kRDFRSCIID);
		var target = BMDS.GetTarget(urlRes, urlArc, true);
		if (target) {
			AllURLs[AllURLsTotal] = target.QueryInterface(kRDFLITIID).Value;
			AllURLsTotal++;
		}
	}
	var URLDuplicates = new Array();
	//var URLDuplicatesParent = new Array();
	var urlcount = 0;
	for (var countB = 0; countB < AllURLs.length;countB++){
		var isDuplicate=isBookmarked(AllURLs[countB]);
	if(version == "1.5"){
		if (isDuplicate && isDuplicate.length > 1 && URLDuplicates.indexOf(AllURLs[countB]) == -1){
			URLDuplicates[urlcount] = AllURLs[countB];
			//URLDuplicatesParent[urlcount] = isDuplicate;
			urlcount++;
		}
	//indexOf not defined in version 1.0 !!!
	} else if  (version == "1.0"){
		if (isDuplicate && isDuplicate.length > 1){
			var alreadyIN = false;
			for (var count=0; count<URLDuplicates.length; count++){
				if (URLDuplicates[count] == AllURLs[countB]) alreadyIN = true;
			}
			if (!alreadyIN){
				URLDuplicates[urlcount] = AllURLs[countB];
				//URLDuplicatesParent[urlcount] = isDuplicate;
				urlcount++;
			}
		}
	}
	}
	URLDuplicates.sort();
	var numpage = Math.ceil(URLDuplicates.length/20);
	if (numpage >0 ){
		for (var countC=0; countC*20<URLDuplicates.length;countC++){
			var message ="";
			try {
				message = DuplicatesBundle.getFormattedString("SearchDup", [URLDuplicates.length, AllURLsTotal, eval(countC+1), numpage]);
			} catch (e) {
				message = DuplicatesBundle.getString("SearchDup-TotalURL")+" "+URLDuplicates.length+" "+DuplicatesBundle.getString("SearchDup-OutOF")+" "+AllURLs.length+" ";
				message += DuplicatesBundle.getString("SearchDup-ListOF")+" "+eval(countC+1)+"/"+numpage+"\n";
			}
			var showResult = URLDuplicates.slice(countC*20,(countC+1)*20);
			message += showResult.join("\n");
			alert(message);
		}
	} else { alert(DuplicatesBundle.getString("SearchDup-none")); }
}


// Replace function addBookmarkForTabBrowser in browser.js
// Add support for isBookmarked()
function addBookmarkForTabBrowser(aTabBrowser, aBookmarkAllTabs, aSelect)
{
	var DuplicatesBundle = document.getElementById('duplicates-strings');
	var tabsInfo = [];
	var currentTabInfo = { name: "", url: "", charset: null };
	if (version == "1.0" && aBookmarkAllTabs) {
		aSelect = aBookmarkAllTabs;
		if (aSelect) aBookmarkAllTabs = true;
	}

	const activeBrowser = aTabBrowser.selectedBrowser;
	const browsers = aTabBrowser.browsers;
	for (var i = 0, j = 0; i < browsers.length; ++i) {
		var webNav = browsers[i].webNavigation;
		var url = webNav.currentURI.spec;
		var name = "";
		var charSet, description;
		try {
			var doc = webNav.document;
			name = doc.title || url;
			charSet = doc.characterSet;
			if (version == "1.5") description = BookmarksUtils.getDescriptionFromDocument(doc);
		} catch (e) {
			name = url;
		}

		var isDuplicate = isBookmarked(url);
		if(isDuplicate) {
			var message = DuplicatesBundle.getString("followingURL")+"\n";
			message += url+"\n";
			message += DuplicatesBundle.getString("inFolder")+"\n";
			message += isDuplicate.join("\n")+"\n";
			message += DuplicatesBundle.getString("alreadyIn-Confirm");

			//var answerOK = confirm(message) || false;
			if(aBookmarkAllTabs) {
				if (! confirm(message)) continue;
			} else if (browsers[i] == activeBrowser) {
				if (! confirm(message)) return false;
			}
		}

		tabsInfo[j] = { name: name, url: url, charset: charSet, description: description };
		if (browsers[i] == activeBrowser) currentTabInfo = tabsInfo[j];
		j++;
	}

	if (tabsInfo.length == 0) {
		alert (DuplicatesBundle.getString("NoNewURL"));
		return false;
	}

	var dialogArgs = currentTabInfo;
	if (version == "1.5" && aBookmarkAllTabs) {
		dialogArgs = { name: gNavigatorBundle.getString("bookmarkAllTabsDefault") };
		dialogArgs.bBookmarkAllTabs = true;
	}


	dialogArgs.objGroup = tabsInfo;
	if(version == "1.5") {
		return openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", ADD_BM_DIALOG_FEATURES, dialogArgs);
	} else if (version == "1.0") {
		return openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", "centerscreen,chrome,dialog,resizable,dependent", currentTabInfo.name, currentTabInfo.url, null, currentTabInfo.charset, "addGroup" + (aSelect ? ",group" : ""), tabsInfo);


	}
}
// Return a chain of URL Parent Folder Name 
function findPath(URLresource){
	var ParentResource = URLresource;
	//var ParentArray = new Array();
	var PathArray = new Array();
	var boucle = true, ParentName;

	while (boucle){
		ParentResource = BMSVC.getParent(ParentResource);
		if (ParentResource) {
			ParentName = BookmarksUtils.getProperty(ParentResource.Value,gNCNS+"Name");
			//ParentArray.push(ParentResource);
			if (ParentName) PathArray.push(ParentName);
		} else {
			boucle = false;
		}
	}
	PathArray.reverse();
	return PathArray.join("-->");
}
